@@ -25,8 +25,6 @@ static byte buf[BUFFER_SIZE] = {};
 
 #define SET_SIZE_FOR_FORMAT_OUTPUT 10
 
-struct linpool *lp;
-
 enum set_type
 {
   SET_TYPE_INT,
@@ -38,24 +36,23 @@ generate_set_sequence(enum set_type type, int len)
 {
   struct adata empty_as_path = {};
   set_sequence = set_sequence_same = set_sequence_higher = set_random = &empty_as_path;
-  lp = lp_new_default(&root_pool);
 
   int i;
   for (i = 0; i < len; i++)
   {
     if (type == SET_TYPE_INT)
     {
-      set_sequence 	  = int_set_add(lp, set_sequence, i);
-      set_sequence_same   = int_set_add(lp, set_sequence_same, i);
-      set_sequence_higher = int_set_add(lp, set_sequence_higher, i + SET_SIZE);
-      set_random   	  = int_set_add(lp, set_random, bt_random());
+      set_sequence 	  = int_set_add(tmp_linpool, set_sequence, i);
+      set_sequence_same   = int_set_add(tmp_linpool, set_sequence_same, i);
+      set_sequence_higher = int_set_add(tmp_linpool, set_sequence_higher, i + SET_SIZE);
+      set_random   	  = int_set_add(tmp_linpool, set_random, bt_random());
     }
     else if (type == SET_TYPE_EC)
     {
-      set_sequence 	  = ec_set_add(lp, set_sequence, i);
-      set_sequence_same   = ec_set_add(lp, set_sequence_same, i);
-      set_sequence_higher = ec_set_add(lp, set_sequence_higher, i + SET_SIZE);
-      set_random   	  = ec_set_add(lp, set_random, (bt_random() << 32 | bt_random()));
+      set_sequence 	  = ec_set_add(tmp_linpool, set_sequence, i);
+      set_sequence_same   = ec_set_add(tmp_linpool, set_sequence_same, i);
+      set_sequence_higher = ec_set_add(tmp_linpool, set_sequence_higher, i + SET_SIZE);
+      set_random   	  = ec_set_add(tmp_linpool, set_random, (bt_random() << 32 | bt_random()));
     }
     else
       bt_abort_msg("This should be unreachable");
@@ -71,7 +68,6 @@ t_set_int_contains(void)
 {
   int i;
 
-  resource_init();
   generate_set_sequence(SET_TYPE_INT, SET_SIZE);
 
   bt_assert(int_set_get_size(set_sequence) == SET_SIZE);
@@ -85,33 +81,29 @@ t_set_int_contains(void)
   for (i = 0; i < SET_SIZE; i++)
     bt_assert_msg(data[i] == i, "(data[i] = %d) == i = %d)", data[i], i);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_int_union(void)
 {
-  resource_init();
   generate_set_sequence(SET_TYPE_INT, SET_SIZE);
 
   const struct adata *set_union;
-  set_union = int_set_union(lp, set_sequence, set_sequence_same);
+  set_union = int_set_union(tmp_linpool, set_sequence, set_sequence_same);
   bt_assert(int_set_get_size(set_union) == SET_SIZE);
   bt_assert(int_set_format(set_union, 0, 2, buf, BUFFER_SIZE) == 0);
 
-  set_union = int_set_union(lp, set_sequence, set_sequence_higher);
+  set_union = int_set_union(tmp_linpool, set_sequence, set_sequence_higher);
   bt_assert_msg(int_set_get_size(set_union) == SET_SIZE*2, "int_set_get_size(set_union) %d, SET_SIZE*2 %d", int_set_get_size(set_union), SET_SIZE*2);
   bt_assert(int_set_format(set_union, 0, 2, buf, BUFFER_SIZE) == 0);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_int_format(void)
 {
-  resource_init();
   generate_set_sequence(SET_TYPE_INT, SET_SIZE_FOR_FORMAT_OUTPUT);
 
   bt_assert(int_set_format(set_sequence, 0, 0, buf, BUFFER_SIZE) == 0);
@@ -125,21 +117,19 @@ t_set_int_format(void)
   bt_assert(int_set_format(set_sequence, 1, 0, buf, BUFFER_SIZE) == 0);
   bt_assert(strcmp(buf, "(0,0) (0,1) (0,2) (0,3) (0,4) (0,5) (0,6) (0,7) (0,8) (0,9)") == 0);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_int_delete(void)
 {
-  resource_init();
   generate_set_sequence(SET_TYPE_INT, SET_SIZE);
 
   const struct adata *deleting_sequence = set_sequence;
   u32 i;
   for (i = 0; i < SET_SIZE; i++)
   {
-    deleting_sequence = int_set_del(lp, deleting_sequence, i);
+    deleting_sequence = int_set_del(tmp_linpool, deleting_sequence, i);
     bt_assert_msg(int_set_get_size(deleting_sequence) == (int) (SET_SIZE-1-i),
 		  "int_set_get_size(deleting_sequence) %d == SET_SIZE-1-i %d",
 		  int_set_get_size(deleting_sequence),
@@ -160,7 +150,6 @@ t_set_ec_contains(void)
 {
   u32 i;
 
-  resource_init();
   generate_set_sequence(SET_TYPE_EC, SET_SIZE);
 
   bt_assert(ec_set_get_size(set_sequence) == SET_SIZE);
@@ -174,62 +163,54 @@ t_set_ec_contains(void)
 //  for (i = 0; i < SET_SIZE; i++)
 //    bt_assert_msg(data[i] == (SET_SIZE-1-i), "(data[i] = %d) == ((SET_SIZE-1-i) = %d)", data[i], SET_SIZE-1-i);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_ec_union(void)
 {
-  resource_init();
   generate_set_sequence(SET_TYPE_EC, SET_SIZE);
 
   const struct adata *set_union;
-  set_union = ec_set_union(lp, set_sequence, set_sequence_same);
+  set_union = ec_set_union(tmp_linpool, set_sequence, set_sequence_same);
   bt_assert(ec_set_get_size(set_union) == SET_SIZE);
   bt_assert(ec_set_format(set_union, 0, buf, BUFFER_SIZE) == 0);
 
-  set_union = ec_set_union(lp, set_sequence, set_sequence_higher);
+  set_union = ec_set_union(tmp_linpool, set_sequence, set_sequence_higher);
   bt_assert_msg(ec_set_get_size(set_union) == SET_SIZE*2, "ec_set_get_size(set_union) %d, SET_SIZE*2 %d", ec_set_get_size(set_union), SET_SIZE*2);
   bt_assert(ec_set_format(set_union, 0, buf, BUFFER_SIZE) == 0);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_ec_format(void)
 {
-  resource_init();
-
   const struct adata empty_as_path = {};
   set_sequence = set_sequence_same = set_sequence_higher = set_random = &empty_as_path;
-  lp = lp_new_default(&root_pool);
 
   u64 i = 0;
-  set_sequence = ec_set_add(lp, set_sequence, i);
+  set_sequence = ec_set_add(tmp_linpool, set_sequence, i);
   for (i = 1; i < SET_SIZE_FOR_FORMAT_OUTPUT; i++)
-    set_sequence = ec_set_add(lp, set_sequence, i + ((i%2) ? ((u64)EC_RO << 48) : ((u64)EC_RT << 48)));
+    set_sequence = ec_set_add(tmp_linpool, set_sequence, i + ((i%2) ? ((u64)EC_RO << 48) : ((u64)EC_RT << 48)));
 
   bt_assert(ec_set_format(set_sequence, 0, buf, BUFFER_SIZE) == 0);
   bt_assert_msg(strcmp(buf, "(unknown 0x0, 0, 0) (ro, 0, 1) (rt, 0, 2) (ro, 0, 3) (rt, 0, 4) (ro, 0, 5) (rt, 0, 6) (ro, 0, 7) (rt, 0, 8) (ro, 0, 9)") == 0,
 		"ec_set_format() returns '%s'", buf);
 
-  rfree(lp);
   return 1;
 }
 
 static int
 t_set_ec_delete(void)
 {
-  resource_init();
   generate_set_sequence(SET_TYPE_EC, SET_SIZE);
 
   const struct adata *deleting_sequence = set_sequence;
   u32 i;
   for (i = 0; i < SET_SIZE; i++)
   {
-    deleting_sequence = ec_set_del(lp, deleting_sequence, i);
+    deleting_sequence = ec_set_del(tmp_linpool, deleting_sequence, i);
     bt_assert_msg(ec_set_get_size(deleting_sequence) == (int) (SET_SIZE-1-i),
 		  "ec_set_get_size(deleting_sequence) %d  == SET_SIZE-1-i %d",
 		  ec_set_get_size(deleting_sequence), SET_SIZE-1-i);
