@@ -17,6 +17,7 @@
 #include "nest/protocol.h"
 #include "nest/route.h"
 #include "nest/attrs.h"
+#include "nest/mpls.h"
 #include "conf/conf.h"
 #include "lib/resource.h"
 #include "lib/string.h"
@@ -1151,6 +1152,19 @@ bgp_attr_known(uint code)
   return (code < ARRAY_SIZE(bgp_attr_table)) && bgp_attr_table[code].name;
 }
 
+const char *
+bgp_attr_name(uint code)
+{
+  return (code < ARRAY_SIZE(bgp_attr_table)) ? bgp_attr_table[code].name : NULL;
+}
+
+void bgp_fix_attr_flags(ea_list *attrs)
+{
+  for (u8 i = 0; i < attrs->count; i++)
+  {
+    attrs->attrs[i].flags = bgp_attr_table[EA_ID(attrs->attrs[i].id)].flags;
+  }
+}
 
 /*
  *	Attribute export
@@ -1494,6 +1508,13 @@ bgp_finish_attrs(struct bgp_parse_state *s, rta *a)
 	       p->cf->local_role == BGP_ROLE_RS_CLIENT))
       bgp_set_attr_u32(&a->eattrs, s->pool, BA_ONLY_TO_CUSTOMER, 0, p->cf->remote_as);
   }
+
+  /* Apply MPLS policy for labeled SAFIs */
+  if (s->mpls && s->proto->p.mpls_channel)
+  {
+    struct mpls_channel *mc = (void *) s->proto->p.mpls_channel;
+    ea_set_attr_u32(&a->eattrs, s->pool, EA_MPLS_POLICY, 0, EAF_TYPE_INT, mc->label_policy);
+  }
 }
 
 
@@ -1834,7 +1855,7 @@ bgp_update_attrs(struct bgp_proto *p, struct bgp_channel *c, rte *e, ea_list *at
 
     /* MULTI_EXIT_DESC attribute - accept only if set in export filter */
     a = bgp_find_attr(attrs0, BA_MULTI_EXIT_DISC);
-    if (a && !(a->fresh))
+    if (a && !a->fresh && !p->cf->allow_med)
       bgp_unset_attr(&attrs, pool, BA_MULTI_EXIT_DISC);
   }
 
