@@ -19,6 +19,13 @@
 #include <sys/mman.h>
 #endif
 
+#ifdef CONFIG_DISABLE_THP
+#include <sys/prctl.h>
+#ifndef PR_SET_THP_DISABLE
+#define PR_SET_THP_DISABLE 41
+#endif
+#endif
+
 long page_size = 0;
 
 #ifdef HAVE_MMAP
@@ -58,6 +65,7 @@ struct free_pages {
 };
 
 static void global_free_pages_cleanup_event(void *);
+static void *alloc_cold_page(void);
 
 static struct free_pages global_free_pages = {
   .min = KEEP_PAGES_MAIN_MIN,
@@ -73,7 +81,7 @@ alloc_sys_page(void)
   void *ptr = mmap(NULL, page_size, PROT_WRITE | PROT_READ, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 
   if (ptr == MAP_FAILED)
-    bug("mmap(%lu) failed: %m", page_size);
+    die("mmap(%ld) failed: %m", (s64) page_size);
 
   return ptr;
 }
@@ -94,7 +102,7 @@ alloc_page(void)
     int err = posix_memalign(&ptr, page_size, page_size);
 
     if (err || !ptr)
-      bug("posix_memalign(%lu) failed", (long unsigned int) page_size);
+      die("posix_memalign(%ld) failed", (s64) page_size);
 
     return ptr;
   }
@@ -114,6 +122,14 @@ alloc_page(void)
 
     return fp;
   }
+  else
+    return alloc_cold_page();
+}
+
+static void *
+alloc_cold_page(void)
+{
+  struct free_pages *fps = &global_free_pages;
 
   /* If there is any free page kept cold, we use that. */
   if (!EMPTY_LIST(fps->empty))
@@ -170,12 +186,7 @@ global_free_pages_cleanup_event(void *data UNUSED)
 
   /* Cleanup may get called when hot free page cache is short of pages. Replenishing. */
   while (fps->cnt / 2 < fps->min)
-  {
-    struct free_page *fp = alloc_sys_page();
-    fp->n = (node) {};
-    add_tail(&fps->pages, &fp->n);
-    fps->cnt++;
-  }
+    free_page(alloc_cold_page());
 
   /* Or the hot free page cache is too big. Moving some pages to the cold free page cache. */
   for (int limit = CLEANUP_PAGES_BULK; limit && (fps->cnt > fps->max / 2); fps->cnt--, limit--)
@@ -218,6 +229,12 @@ global_free_pages_cleanup_event(void *data UNUSED)
 void
 resource_sys_init(void)
 {
+#ifdef CONFIG_DISABLE_THP
+  /* Disable transparent huge pages, they do not work properly with madvice(MADV_DONTNEED) */
+  if (prctl(PR_SET_THP_DISABLE,  (unsigned long) 1,  (unsigned long) 0,  (unsigned long) 0,  (unsigned long) 0) < 0)
+    log(L_WARN "Cannot disable transparent huge pages: prctl(PR_SET_THP_DISABLE) failed: %m");
+#endif
+
 #ifdef HAVE_MMAP
   ASSERT_DIE(global_free_pages.cnt == 0);
 
